@@ -19,7 +19,7 @@
 #define DOWNLOAD_URL "http://symless.com/?source=gui"
 #define HELP_URL     "http://symless.com/help?source=gui"
 
-#include <iostream>
+#include <array>
 
 #include "MainWindow.h"
 
@@ -112,8 +112,6 @@ MainWindow::MainWindow (AppConfig& appConfig,
     m_pSynergy(NULL),
     m_SynergyState(synergyDisconnected),
     m_ServerConfig(5, 3, m_AppConfig->screenName(), this),
-    m_pTrayIcon(NULL),
-    m_pTrayIconMenu(NULL),
     m_AlreadyHidden(false),
     m_pMenuBar(NULL),
     m_pMenuFile(NULL),
@@ -122,10 +120,9 @@ MainWindow::MainWindow (AppConfig& appConfig,
     m_pMenuHelp(NULL),
     m_pCancelButton(NULL),
     m_ExpectedRunningState(kStopped),
-    m_pSslCertificate(NULL),
     m_SecureSocket(false)
 {
-#ifndef SYNERGY_ENTERPRISE
+#if !defined(SYNERGY_ENTERPRISE) && defined(SYNERGY_AUTOCONFIG)
     m_pZeroconf = new Zeroconf(this);
 #endif
 
@@ -137,7 +134,9 @@ MainWindow::MainWindow (AppConfig& appConfig,
 
     m_pWidgetUpdate->hide();
     m_VersionChecker.setApp(appPath(appConfig.synergycName()));
+    
     m_pLabelScreenName->setText(getScreenName());
+    connect(m_AppConfig, SIGNAL(screenNameChanged()), this, SLOT(updateScreenName()));
     m_pLabelIpAddresses->setText(getIPAddresses());
 
 #if defined(Q_OS_WIN)
@@ -203,7 +202,7 @@ MainWindow::MainWindow (AppConfig& appConfig,
     m_pActivate->setVisible(false);
 #endif
 
-#ifndef SYNERGY_ENTERPRISE
+#if !defined(SYNERGY_ENTERPRISE) && defined(SYNERGY_AUTOCONFIG)
     updateZeroconfService();
 
     addZeroconfServer(m_AppConfig->autoConfigServer());
@@ -219,16 +218,27 @@ MainWindow::~MainWindow()
         stopDesktop();
     }
 
-#ifndef SYNERGY_ENTERPRISE
+#if !defined(SYNERGY_ENTERPRISE) && defined(SYNERGY_AUTOCONFIG)
     delete m_pZeroconf;
 #endif
-
-    delete m_pSslCertificate;
 }
 
 void MainWindow::open()
 {
-    createTrayIcon();
+    std::array<QAction *, 7> trayMenu = {
+        m_pActionStartSynergy,
+        m_pActionStopSynergy,
+        nullptr,
+        m_pActionMinimize,
+        m_pActionRestore,
+        nullptr,
+        m_pActionQuit
+    };
+
+    m_trayIcon.create(trayMenu, [this](QObject const *o, const char *s) {
+        connect(o, s, this, SLOT(trayActivated(QSystemTrayIcon::ActivationReason)));
+        setIcon(synergyDisconnected);
+    });
 
     if (appConfig().getAutoHide()) {
         hide();
@@ -251,31 +261,6 @@ void MainWindow::setStatus(const QString &status)
     m_pStatusLabel->setText(status);
 }
 
-void MainWindow::createTrayIcon()
-{
-    m_pTrayIconMenu = new QMenu(this);
-
-    m_pTrayIconMenu->addAction(m_pActionStartSynergy);
-    m_pTrayIconMenu->addAction(m_pActionStopSynergy);
-    m_pTrayIconMenu->addSeparator();
-
-    m_pTrayIconMenu->addAction(m_pActionMinimize);
-    m_pTrayIconMenu->addAction(m_pActionRestore);
-    m_pTrayIconMenu->addSeparator();
-    m_pTrayIconMenu->addAction(m_pActionQuit);
-
-    m_pTrayIcon = new QSystemTrayIcon(this);
-    m_pTrayIcon->setContextMenu(m_pTrayIconMenu);
-    m_pTrayIcon->setToolTip("Synergy");
-
-    connect(m_pTrayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
-            this, SLOT(trayActivated(QSystemTrayIcon::ActivationReason)));
-
-    setIcon(synergyDisconnected);
-
-    m_pTrayIcon->show();
-}
-
 void MainWindow::retranslateMenuBar()
 {
     m_pMenuFile->setTitle(tr("&File"));
@@ -351,14 +336,12 @@ void MainWindow::saveSettings()
     appConfig().setConfigFile(m_pLineEditConfigFile->text());
     appConfig().setServerHostname(m_pLineEditHostname->text());
 
-
-    //Save everything
+    /* Save everything */
     GUI::Config::ConfigWriter::make()->globalSave();
-
 }
 
 void MainWindow::zeroConfToggled() {
-#ifndef SYNERGY_ENTERPRISE
+#if !defined(SYNERGY_ENTERPRISE) && defined(SYNERGY_AUTOCONFIG)
     updateZeroconfService();
 
     addZeroconfServer(m_AppConfig->autoConfigServer());
@@ -366,12 +349,13 @@ void MainWindow::zeroConfToggled() {
     updateAutoConfigWidgets();
 #endif
 }
-void MainWindow::setIcon(qSynergyState state)
+
+void MainWindow::setIcon(qSynergyState state) const
 {
     QIcon icon;
 
 #ifdef Q_OS_MAC
-    if (isOSXInterfaceStyleDark())
+    if (isOSXUseDarkIcons())
         icon.addFile(synergyDarkIconFiles[state]);
     else
         icon.addFile(synergyLightIconFiles[state]);
@@ -379,8 +363,7 @@ void MainWindow::setIcon(qSynergyState state)
     icon.addFile(synergyDefaultIconFiles[state]);
 #endif
 
-    if (m_pTrayIcon)
-        m_pTrayIcon->setIcon(icon);
+    m_trayIcon.set(icon);
 }
 
 void MainWindow::trayActivated(QSystemTrayIcon::ActivationReason reason)
@@ -608,6 +591,8 @@ void MainWindow::clearLog()
 
 void MainWindow::startSynergy()
 {
+    saveSettings();
+
 #ifndef SYNERGY_ENTERPRISE
     SerialKey serialKey = m_LicenseManager->serialKey();
     if (!serialKey.isValid()) {
@@ -766,7 +751,7 @@ bool MainWindow::clientArgs(QStringList& args, QString& app)
         args << "--log" << appConfig().logFilenameCmd();
     }
 
-#ifndef SYNERGY_ENTERPRISE
+#if !defined(SYNERGY_ENTERPRISE) && defined(SYNERGY_AUTOCONFIG)
     // check auto config first, if it is disabled or no server detected,
     // use line edit host name if it is not empty
     if (appConfig().autoConfig()) {
@@ -787,7 +772,7 @@ bool MainWindow::clientArgs(QStringList& args, QString& app)
 
     if (m_pLineEditHostname->text().isEmpty())
     {
-#ifndef SYNERGY_ENTERPRISE
+#if !defined(SYNERGY_ENTERPRISE) && defined(SYNERGY_AUTOCONFIG)
         //check if autoconfig mode is enabled
         if (!appConfig().autoConfig())
         {
@@ -798,7 +783,7 @@ bool MainWindow::clientArgs(QStringList& args, QString& app)
                 tr("Please fill in a hostname for the synergy client to connect to."));
             return false;
 
-#ifndef SYNERGY_ENTERPRISE
+#if !defined(SYNERGY_ENTERPRISE) && defined(SYNERGY_AUTOCONFIG)
         }
         else
         {
@@ -847,7 +832,7 @@ QString MainWindow::configFilename()
     return filename;
 }
 
-QString MainWindow::address()
+QString MainWindow::address() const
 {
     QString i = appConfig().networkInterface();
     return (!i.isEmpty() ? i : "") + ":" + QString::number(appConfig().port());
@@ -1235,7 +1220,7 @@ void MainWindow::on_m_pActionHelp_triggered()
 
 void MainWindow::updateZeroconfService()
 {
-#ifndef SYNERGY_ENTERPRISE
+#if !defined(SYNERGY_ENTERPRISE) && defined(SYNERGY_AUTOCONFIG)
 
     // reset the server list in case one has gone away.
     // it'll be re-added after the zeroconf service restarts.
@@ -1260,8 +1245,6 @@ void MainWindow::updateAutoConfigWidgets()
 
         m_pLabelServerName->hide();
         m_pLineEditHostname->hide();
-
-        m_pWidgetAutoConfig->hide();
     }
     else {
         m_pLabelServerName->show();
@@ -1269,12 +1252,6 @@ void MainWindow::updateAutoConfigWidgets()
 
         m_pLabelAutoDetected->hide();
         m_pComboServerList->hide();
-
-#ifndef SYNERGY_ENTERPRISE
-        m_pWidgetAutoConfig->show();
-#else
-        m_pWidgetAutoConfig->hide();
-#endif
     }
 }
 
@@ -1351,8 +1328,6 @@ int MainWindow::raiseActivationDialog()
     }
     ActivationDialog activationDialog (this, appConfig(), licenseManager());
     m_ActivationDialogRunning = true;
-    connect (&activationDialog, SIGNAL(finished(int)),
-             this, SLOT(on_activationDialogFinish()), Qt::QueuedConnection);
     int result = activationDialog.exec();
     m_ActivationDialogRunning = false;
     if (!m_PendingClientNames.empty()) {
@@ -1402,11 +1377,6 @@ void MainWindow::secureSocket(bool secureSocket)
     }
 }
 
-void MainWindow::on_m_pLabelAutoConfig_linkActivated(const QString &)
-{
-    m_pActionSettings->trigger();
-}
-
 void MainWindow::on_m_pComboServerList_currentIndexChanged(const QString &server)
 {
     appConfig().setAutoConfigServer(server);
@@ -1417,3 +1387,8 @@ void MainWindow::windowStateChanged()
     if (windowState() == Qt::WindowMinimized && appConfig().getMinimizeToTray())
         hide();
 }
+
+void MainWindow::updateScreenName() 
+{
+    m_pLabelScreenName->setText(getScreenName());
+}
